/**
 * Объект для работы по SOAP протоколу
 * @constructor
 */
MYSOAP = function()
{
	var AuthXMLTemplate  = AuthTemplate;
	//var AuthXMLTemplate2 = IR.Import("XMLDATA.js").AuthXMLTemplate2;
	var DATA = Data;
	var XMLTemplate = XMLTemplate;
	this.DATA = DATA;
	var MessageUUID = new UUIDGenerator();
	
	/**
	*Получить xml запроса времени камеры
	*Не требует авторизации
	*/
	this.GetSystemDateAndTime = function ()
	{
		return getWrap(DATA.GetSystemDateAndTime.Data);
	};
	/**
	*Получить xml запроса Scopes
	*Требует авторизации, поэтому на входе объект AuthData
	*/
	this.GetCapabilities = function (in_oAuthData)
	{
		//return getAuthWrap(DATA.GetCapabilities.Data, in_oAuthData);
        var Xml = getAuthHeader(in_oAuthData);
        Xml["s:Envelope"]["s:Body"] = {};
        Xml["s:Envelope"]["s:Body"].GetCapabilities = {};
        Xml["s:Envelope"]["s:Body"].GetCapabilities["@xmlns"] = "http://www.onvif.org/ver10/device/wsdl";
        Xml["s:Envelope"]["s:Body"].GetCapabilities.Category = "All";
        return Xml.toXMLString();
	};//PTZContinuousMove
    /**
     * A PTZ-capable device shall return all available PTZConfigurations through the
     * GetConfigurations operation.
     * @param in_oAuthData
     * @returns {string} //This is an empty message.
     */
    this.GetConfigurations = function (in_oAuthData)
    {
        var Xml = getAuthHeader(in_oAuthData);
        Xml["s:Envelope"]["s:Body"] = {};
        Xml["s:Envelope"]["s:Body"].GetConfigurations = {};
        Xml["s:Envelope"]["s:Body"].GetConfigurations["@xmlns"] = "http://www.onvif.org/ver20/ptz/wsdl";
        return Xml.toXMLString();
    };
    /**
     *Получить xml запроса Configuration Options
     *Требует авторизации, поэтому на входе объект @in_oAuthData
     *И токен конфигурации @in_sToken
     */
    this.GetConfigurationOptions = function (in_oAuthData, in_sToken)
    {
        var Xml = getAuthHeader(in_oAuthData);
        Xml["s:Envelope"]["@xmlns:tptz"] = "http://www.onvif.org/ver20/ptz/wsdl";
        Xml["s:Envelope"]["s:Body"]["tptz:GetConfigurationOptions"] = {};
        Xml["s:Envelope"]["s:Body"]["tptz:GetConfigurationOptions"]["@xmlns"] = "http://www.onvif.org/ver20/ptz/wsdl";
        Xml["s:Envelope"]["s:Body"]["tptz:GetConfigurationOptions"]["tptz:ConfigurationToken"] = in_sToken;
        return Xml.toXMLString();
    };
    /**
     *Получить xml запроса PTZ Continuous Move
     *Требует авторизации, поэтому на входе объект @in_oAuthData
     *Так же нужен токен видеопотока @in_sToken
     *Так же нужyны коэфиценты ускорения по осям @in_x, @in_y
     */
    this.PTZContinuousMove = function (in_oAuthData, in_sToken, in_x, in_y)
    {
        var Xml = getAuthHeader(in_oAuthData);
        Xml["s:Envelope"]["@xmlns:tptz"] = "http://www.onvif.org/ver20/ptz/wsdl";
        Xml["s:Envelope"]["s:Body"]["tptz:ContinuousMove"] = {};
        Xml["s:Envelope"]["s:Body"]["tptz:ContinuousMove"]["tptz:ProfileToken"] = in_sToken;
        Xml["s:Envelope"]["s:Body"]["tptz:ContinuousMove"]["tptz:Velocity"] = {};
        Xml["s:Envelope"]["s:Body"]["tptz:ContinuousMove"]["tptz:Velocity"]["tt:PanTilt"] = {};
        Xml["s:Envelope"]["s:Body"]["tptz:ContinuousMove"]["tptz:Velocity"]["tt:PanTilt"]["@x"] = in_x;
        Xml["s:Envelope"]["s:Body"]["tptz:ContinuousMove"]["tptz:Velocity"]["tt:PanTilt"]["@y"] = in_y;
        //Xml["s:Envelope"]["s:Body"]["tptz:ContinuousMove"]["tptz:Velocity"]["tt:Zoom"] = {};
        //Xml["s:Envelope"]["s:Body"]["tptz:ContinuousMove"]["tptz:Velocity"]["tt:Zoom"]["@x"] = 0;
        return Xml.toXMLString();
    };
    /**
     *Получить xml запроса PTZ Zoom, то же самое, что и PTZContinuousMove, но с 0ми по х и у
     *Требует авторизации, поэтому на входе объект @in_oAuthData
     *Так же нужен токен видеопотока @in_sToken
     *Так же нужeн коэфицент ускорения увиличения @in_zoom
     */
    this.PTZZoom = function (in_oAuthData, in_sToken, in_zoom)
    {
        var Xml = getAuthHeader(in_oAuthData);
        Xml["s:Envelope"]["@xmlns:tptz"] = "http://www.onvif.org/ver20/ptz/wsdl";
        Xml["s:Envelope"]["s:Body"]["tptz:ContinuousMove"] = {};
        Xml["s:Envelope"]["s:Body"]["tptz:ContinuousMove"]["tptz:ProfileToken"] = in_sToken;
        Xml["s:Envelope"]["s:Body"]["tptz:ContinuousMove"]["tptz:Velocity"] = {};
        Xml["s:Envelope"]["s:Body"]["tptz:ContinuousMove"]["tptz:Velocity"]["tt:PanTilt"] = {};
        Xml["s:Envelope"]["s:Body"]["tptz:ContinuousMove"]["tptz:Velocity"]["tt:PanTilt"]["@x"] = 0;
        Xml["s:Envelope"]["s:Body"]["tptz:ContinuousMove"]["tptz:Velocity"]["tt:PanTilt"]["@y"] = 0;
        Xml["s:Envelope"]["s:Body"]["tptz:ContinuousMove"]["tptz:Velocity"]["tt:Zoom"] = {};
        Xml["s:Envelope"]["s:Body"]["tptz:ContinuousMove"]["tptz:Velocity"]["tt:Zoom"]["@x"] = in_zoom;
        return Xml.toXMLString();
    };
    /**
     *         НЕ ИСПОЛЬЗУЕМАЯ
     *Получить xml запроса PTZ Absolute Move
     *Требует авторизации, поэтому на входе объект @in_oAuthData
     */
    this.PTZAbsoluteMove = function (in_oAuthData, in_sToken, in_x, in_y)
    {
        var Xml = getAuthHeader(in_oAuthData);
        Xml["s:Envelope"]["@xmlns:tptz"] = "http://www.onvif.org/ver20/ptz/wsdl";
        Xml["s:Envelope"]["s:Body"]["tptz:AbsoluteMove"] = {};
        Xml["s:Envelope"]["s:Body"]["tptz:AbsoluteMove"]["tptz:ProfileToken"] = in_sToken;
        Xml["s:Envelope"]["s:Body"]["tptz:AbsoluteMove"]["tptz:Position "] = {};
        //Xml["s:Envelope"]["s:Body"]["tptz:AbsoluteMove"]["tptz:Speed"]["tt:PanTilt"] = {};
        //Xml["s:Envelope"]["s:Body"]["tptz:AbsoluteMove"]["tptz:Speed"]["tt:PanTilt"]["@x"] = in_x;
        //Xml["s:Envelope"]["s:Body"]["tptz:AbsoluteMove"]["tptz:Speed"]["tt:PanTilt"]["@y"] = in_y;
        //Xml["s:Envelope"]["s:Body"]["tptz:ContinuousMove"]["tptz:Velocity"]["tt:Zoom"] = {};
        //Xml["s:Envelope"]["s:Body"]["tptz:ContinuousMove"]["tptz:Velocity"]["tt:Zoom"]["@x"] = 0;
        return Xml.toXMLString();
    };
    /**
     *Получить xml запроса PTZ Stop
     *Требует авторизации, поэтому на входе объект @in_oAuthData
	 *Так же нужен токен видеопотока @in_sToken
     */
    this.PTZStop = function (in_oAuthData, in_sToken)
    {
        var Xml = getAuthHeader(in_oAuthData);
        Xml["s:Envelope"]["@xmlns:tptz"] = "http://www.onvif.org/ver20/ptz/wsdl";
        Xml["s:Envelope"]["s:Body"]["tptz:Stop"] = {};
        Xml["s:Envelope"]["s:Body"]["tptz:Stop"]["tptz:ProfileToken"] = in_sToken;
        Xml["s:Envelope"]["s:Body"]["tptz:Stop"]["tptz:Velocity"] = {};
        Xml["s:Envelope"]["s:Body"]["tptz:Stop"]["tptz:Velocity"]["tt:PanTilt"] = true;
        Xml["s:Envelope"]["s:Body"]["tptz:Stop"]["tptz:Velocity"]["tt:Zoom"] = false;
        return Xml.toXMLString();
    };
    /**
     *Получить xml запроса Sources
     *Требует авторизации, поэтому на входе объект AuthData
     */
    this.GetVideoSources = function (in_oAuthData)
    {
        //return getAuthWrap(DATA.GetCapabilities.Data, in_oAuthData);
        var Xml = getAuthHeader(in_oAuthData);
        Xml["s:Envelope"]["s:Body"] = {};
        Xml["s:Envelope"]["s:Body"].GetVideoSources = {};
        Xml["s:Envelope"]["s:Body"].GetVideoSources["@xmlns"] = "http://www.onvif.org/ver10/media/wsdl";
        return Xml.toXMLString();

    };
	/**
	*Получить xml запроса Scopes
	*Требует авторизации, поэтому на входе объект AuthData
	*/
	this.GetScopes = function (in_oAuthData)
	{
		return getAuthWrap(DATA.GetScopes.Data, in_oAuthData);
	};
    /**
     *Получить xml запроса Services
     *Требует авторизации, поэтому на входе объект AuthData
     */
    this.GetServices = function (in_oAuthData)
    {
        //return getAuthWrap(DATA.GetScopes.Data, in_oAuthData);
        var Xml = getAuthHeader(in_oAuthData);
        Xml["s:Envelope"]["s:Body"] = {};
        Xml["s:Envelope"]["s:Body"].GetServices = {}; //
        Xml["s:Envelope"]["s:Body"].GetServices["@xmlns"] = "http://www.onvif.org/ver10/device/wsdl";
        Xml["s:Envelope"]["s:Body"].GetServices.IncludeCapability = false;
        return Xml.toXMLString();
    };
    /**
     *Получить xml запроса PullPointSubscription
     *Требует авторизации, поэтому на входе объект AuthData
     */
    this.GetPullPointSubscription= function (in_oAuthData)
    {
       var Xml = getAuthHeader(in_oAuthData);
        Xml["s:Envelope"]["@xmlns:a"] = "http://www.w3.org/2005/08/addressing";

        Xml["s:Envelope"]["s:Header"]["a:Action"] = {};
        Xml["s:Envelope"]["s:Header"]["a:Action"]["@s:mustUnderstand"] = 1;
        Xml["s:Envelope"]["s:Header"]["a:Action"] = "http://www.onvif.org/ver10/events/wsdl/EventPortType/CreatePullPointSubscriptionRequest";
        Xml["s:Envelope"]["s:Header"]["a:MessageID"] = {};
        Xml["s:Envelope"]["s:Header"]["a:MessageID"] = MessageUUID.generate();
        Xml["s:Envelope"]["s:Header"]["a:ReplyTo"] = {};
        Xml["s:Envelope"]["s:Header"]["a:ReplyTo"]["a:Address"]  = {};
        Xml["s:Envelope"]["s:Header"]["a:ReplyTo"]["a:Address"]  = "http://192.168.0.73:1090/onvif/device_service";

        Xml["s:Envelope"]["s:Header"]["a:To"] = {};
        Xml["s:Envelope"]["s:Header"]["a:To"]["@s:mustUnderstand"] = 1;
        Xml["s:Envelope"]["s:Header"]["a:To"] = "http://192.168.0.73:1090/onvif/device_service";

        Xml["s:Envelope"]["s:Body"] = {};
        Xml["s:Envelope"]["s:Body"].CreatePullPointSubscription = {}; //
        Xml["s:Envelope"]["s:Body"].CreatePullPointSubscription["@xmlns"]  = "http://www.onvif.org/ver10/events/wsdl";
        Xml["s:Envelope"]["s:Body"].CreatePullPointSubscription.InitialTerminationTime = "PT600S";
        return Xml.toXMLString();
    };
	/**
	*Получить xml запроса видео потока
	*Требует авторизации, поэтому на входе объект AuthData
	*/
	this.GetSnapshotUri = function (in_oAuthData, in_sToken, in_sStream)
	{//GetStreamUri xmlns="http://www.onvif.org/ver10/media/wsdl"
		var Xml = getAuthHeader(in_oAuthData);

        Xml["s:Envelope"]["s:Body"] = {};
		Xml["s:Envelope"]["s:Body"].GetSnapshotUri = {};
        Xml["s:Envelope"]["s:Body"].GetSnapshotUri["@xmlns"] = "http://www.onvif.org/ver10/media/wsdl";
		Xml["s:Envelope"]["s:Body"].GetSnapshotUri.ProfileToken = {};
		Xml["s:Envelope"]["s:Body"].GetSnapshotUri.ProfileToken = in_sToken;
        return Xml.toXMLString();
        /*
       Xml["s:Envelope"]["s:Body"] = {};
       Xml["s:Envelope"]["s:Body"].GetStreamUri = {};
       Xml["s:Envelope"]["s:Body"].GetStreamUri["StreamSetup"] = {};
       Xml["s:Envelope"]["s:Body"].GetStreamUri["StreamSetup"]["Stream"] = {};
       Xml["s:Envelope"]["s:Body"].GetStreamUri["StreamSetup"]["Stream"]["@xmlns"] = 'mlns="http://www.onvif.org/ver10/schema"';
       Xml["s:Envelope"]["s:Body"].GetStreamUri["StreamSetup"]["Stream"] = "RTP-Unicast";
       Xml["s:Envelope"]["s:Body"].GetStreamUri["StreamSetup"]["Transport"] = {};
       Xml["s:Envelope"]["s:Body"].GetStreamUri["StreamSetup"]["Transport"]["@xmlns"] = 'mlns="http://www.onvif.org/ver10/schema"';
       Xml["s:Envelope"]["s:Body"].GetStreamUri["StreamSetup"]["Transport"]["Protocol"] = "RTSP";
       Xml["s:Envelope"]["s:Body"].GetStreamUri["ProfileToken"] = "quality_h264";
       */
	};

    this.GetStreamUri = function (in_oAuthData, in_sToken, in_sStream)
    {//GetStreamUri xmlns="http://www.onvif.org/ver10/media/wsdl"
        var Xml = getAuthHeader(in_oAuthData);

        Xml["s:Envelope"]["s:Body"] = {};
        Xml["s:Envelope"]["s:Body"].GetStreamUri = {}; //
        Xml["s:Envelope"]["s:Body"].GetStreamUri["@xmlns"] = "http://www.onvif.org/ver10/media/wsdl";
        Xml["s:Envelope"]["s:Body"].GetStreamUri.StreamSetup = {};
        Xml["s:Envelope"]["s:Body"].GetStreamUri.StreamSetup.Stream = {};
        Xml["s:Envelope"]["s:Body"].GetStreamUri.StreamSetup.Stream["@xmlns"] = "http://www.onvif.org/ver10/schema";
        Xml["s:Envelope"]["s:Body"].GetStreamUri.StreamSetup.Stream = "RTP-Unicast";
        Xml["s:Envelope"]["s:Body"].GetStreamUri.StreamSetup.Transport = {};
        Xml["s:Envelope"]["s:Body"].GetStreamUri.StreamSetup.Transport["@xmlns"] = "http://www.onvif.org/ver10/schema";
        Xml["s:Envelope"]["s:Body"].GetStreamUri.StreamSetup.Transport.Protocol = "RTSP";
        Xml["s:Envelope"]["s:Body"].GetStreamUri.ProfileToken = in_sToken;
        return Xml.toXMLString();
    };

    this.GetProfile = function (in_oAuthData, in_sProfile)
    {
        var Xml = getAuthHeader(in_oAuthData);

        Xml["s:Envelope"]["s:Body"] = {};
        Xml["s:Envelope"]["s:Body"].GetProfile = {}; //
        Xml["s:Envelope"]["s:Body"].GetProfile["@xmlns"] = "http://www.onvif.org/ver10/media/wsdl";
        Xml["s:Envelope"]["s:Body"].GetProfile.ProfileToken = {};
        Xml["s:Envelope"]["s:Body"].GetProfile.ProfileToken = in_sProfile;
        return Xml.toXMLString().replace(/\r|\n/g, '');
    };
    this.wsdlGetProfile = function (in_oAuthData, in_sProfile)
    {
        var Xml = getAuthHeader(in_oAuthData);

        Xml["s:Envelope"]["s:Body"] = {};
        Xml["s:Envelope"]["s:Body"].GetProfile = {}; //
        Xml["s:Envelope"]["s:Body"].GetProfile["@xmlns"] = "http://www.onvif.org/ver10/media/wsdl";
        Xml["s:Envelope"]["s:Body"].GetProfile.ProfileToken = {};
        Xml["s:Envelope"]["s:Body"].GetProfile.ProfileToken = in_sProfile;
        return Xml.toXMLString().replace(/\r|\n/g, '');
    };
	/**
	*Получить xml запроса настрйоки протокола потока
	*Требует авторизации, поэтому на входе объект AuthData
	*/
	this.GetProfiles = function (in_oAuthData)
	{
		return getAuthWrap(DATA.GetProfiles.Data, in_oAuthData);
	};
	/**
	*Получить xml запроса настрйоки протокола потока
	*Требует авторизации, поэтому на входе объект AuthData
	*/
	this.GetNetworkProtocols = function (in_oAuthData)
	{
		return getAuthWrap(DATA.GetNetworkProtocols.Data, in_oAuthData);
	};
	/**
	*Получить xml запроса сетевых интерфесов
	*Требует авторизации, поэтому на входе объект AuthData
	*/
	this.GetNetInterfaces = function (in_oAuthData)
	{
		return getAuthWrap(DATA.GetNetInterfaces.Data, in_oAuthData);
	};
	/**
	*Обернуть в XML с авторизацией
	*На входе, то что нужно обернуть
	*/
	function getAuthWrap(in_sData, in_oAuthData)
	{
		var Xml = getAuthHeader(in_oAuthData);
		Xml["s:Envelope"]["s:Body"] = {};
		Xml["s:Envelope"]["s:Body"][in_sData] = {};
      return Xml.toXMLString();
	}
	/**
	*Формирование заголовка с авторизацией
	*на основе объекта AuthData (Время синхронизации, логин, пароль)
	*на входе объект AuthData
	*/

	function getAuthHeader(in_oAuthData)
	{
        var NowAuthXMLTemplate = new XML(AuthXMLTemplate);

		var l_sTime = getUTCPreformedTime(in_oAuthData);
        var l_sNonce = getNonce();
		var l_sHexPassSha = IR.CalculateHash(IR.HASH_SHA1, l_sNonce + l_sTime + in_oAuthData.Password);
        var l_sDigistPassSha = hexStrtoDigist(l_sHexPassSha);
        NowAuthXMLTemplate["s:Envelope"]["s:Header"]["Security"] = {};
        NowAuthXMLTemplate["s:Envelope"]["s:Header"]["Security"]["@s:mustUnderstand"] = 1;
        NowAuthXMLTemplate["s:Envelope"]["s:Header"]["Security"]["@xmlns"] = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd";
        NowAuthXMLTemplate["s:Envelope"]["s:Header"]["Security"]["UsernameToken"] = {};
        NowAuthXMLTemplate["s:Envelope"]["s:Header"]["Security"]["UsernameToken"]["Username"] = in_oAuthData.Login;
        NowAuthXMLTemplate["s:Envelope"]["s:Header"]["Security"]["UsernameToken"]["Password"] = {};
        NowAuthXMLTemplate["s:Envelope"]["s:Header"]["Security"]["UsernameToken"]["Password"]["@Type"] = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordDigest";
        NowAuthXMLTemplate["s:Envelope"]["s:Header"]["Security"]["UsernameToken"]["Password"] = Base64.encode(l_sDigistPassSha);
        NowAuthXMLTemplate["s:Envelope"]["s:Header"]["Security"]["UsernameToken"]["Nonce"] = {};
        NowAuthXMLTemplate["s:Envelope"]["s:Header"]["Security"]["UsernameToken"]["Nonce"]["@EncodingType"] = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-soap-message-security-1.0#Base64Binary";
        NowAuthXMLTemplate["s:Envelope"]["s:Header"]["Security"]["UsernameToken"]["Nonce"] = Base64.encode(l_sNonce);
        NowAuthXMLTemplate["s:Envelope"]["s:Header"]["Security"]["UsernameToken"]["Created"] = {};
        NowAuthXMLTemplate["s:Envelope"]["s:Header"]["Security"]["UsernameToken"]["Created"]["@xmlns"] = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd";
        NowAuthXMLTemplate["s:Envelope"]["s:Header"]["Security"]["UsernameToken"]["Created"] = l_sTime;
      return NowAuthXMLTemplate;
	}
	/**
	*Обернуть в XML без авторизации
	*На входе, то что нужно обернуть
	*/
	function getWrap(in_sData)
	{
		this.XMLTemplate["s:Envelope"]["s:Body"][in_sData] = {};
		return this.XMLTemplate.toXMLString();
	}
	/**
	*Вычисление нужного времени из разницы времён
	*Нужна как соль для хэша авторизации
	*на входе объект AuthData в нем сохранено время
	*(Нужное время = Время при Синхронизации на камере + (Текущее время - Время Синхронизации с камерой))
	*/
	function getUTCPreformedTime(in_oAuthData)
	{
		var l_sNowDeviceDate = new Date().getTime();
		var l_sDiffTime = l_sNowDeviceDate - in_oAuthData.ModuleTimeOnAuth;
		var l_sNeededTime = in_oAuthData.CamTimeOnAuth + l_sDiffTime;
		return prepareDateString(new Date(l_sNeededTime));
	}
	/**
	*Возвращает строку времени вида 2010-09-16T07:50:45Z
	*Нужна как соль для хэша авторизации
	*на входе объект Date
	*/
	function prepareDateString(in_oDate)
	{
		var l_nMonth = prepareDateDigit(in_oDate.getMonth());
		var l_nDate = prepareDateDigit(in_oDate.getDate());
		var l_nHours = prepareDateDigit(in_oDate.getHours());
		var l_nMinutes = prepareDateDigit(in_oDate.getMinutes());
		var l_nSeconds = prepareDateDigit(in_oDate.getSeconds());
		
		return (in_oDate.getFullYear() + "-" + l_nMonth + "-" + l_nDate + "T" + l_nHours + ":" + l_nMinutes + ":" + l_nSeconds + "Z")
	}
	/**
	*Подготовка чисел до 10 к виду 01, 02 и т.д.
	*На входе число in_nDateDigit
	*/
	function prepareDateDigit(in_nDateDigit)
	{		
		if(in_nDateDigit < 10)
			in_nDateDigit = "0" + in_nDateDigit;
		return in_nDateDigit;
	}
	/**
	*Генератор случайных hexСтрок
	*вида 92e9a3d6a2189c073482c4cdbc7c3cdd
	*На входе ничего
	*/
	function getNonce() {
		var l_hexNonce = '';
		
		for(var i=0; i<32; i++) 
		{
			l_hexNonce += getRandomInt().toString(16);
		}
			
		function getRandomInt() {
			var l_sMin = 0;
			var l_sMax = 16;
			return Math.floor(Math.random() * (l_sMax - l_sMin)) + l_sMin;
		}
		return l_hexNonce;
	}
	/**
	*Получение digist строки 5j+y░LTWM┬НFц9T(л 
	*вместо hex 356a192b7913b04c54574d18c28d46e6395428ab
	*На входе in_hexStr
	*/
	function hexStrtoDigist(in_hexStr) 
	{
		var l_nLength = in_hexStr.length - 1;
		
		var l_digestStr = '';
		
		for(var i=0; i<l_nLength; i+=2) 
		{
			var sim = in_hexStr.slice(i, i+2);
		
			l_digestStr += String.fromCharCode(parseInt(sim, 16));
		}
		return l_digestStr;
	}
	//Простой Base64 без конвертации в UTF-8
	var Base64 = {
		_keyStr: "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",

		encode: function(input) 
		{
			var output = "";
			var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
			var i = 0;

			while (i < input.length) {

				chr1 = input.charCodeAt(i++);
				chr2 = input.charCodeAt(i++);
				chr3 = input.charCodeAt(i++);

				enc1 = chr1 >> 2;
				enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
				enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
				enc4 = chr3 & 63;

				if (isNaN(chr2)) 
				{
					enc3 = enc4 = 64;
				} else if (isNaN(chr3)) 
				{
					enc4 = 64;
				}
				output = output + this._keyStr.charAt(enc1) + this._keyStr.charAt(enc2) + this._keyStr.charAt(enc3) + this._keyStr.charAt(enc4);
			}
         
			return output;
		}
	}

};
/**
 * Fast UUID generator, RFC4122 version 4 compliant.
 * @author Jeff Ward (jcward.com).
 * @license MIT license
 * @link http://stackoverflow.com/questions/105034/how-to-create-a-guid-uuid-in-javascript/21963136#21963136
 * Объект генератора uuid
 * @returns {string} //xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx
 * @constructor
 */
var UUIDGenerator = function() {
    var self = {};
    var lut = []; for (var i=0; i<256; i++) { lut[i] = (i<16?'0':'')+(i).toString(16); }
    self.generate = function() {
        var d0 = Math.random()*0xffffffff|0;
        var d1 = Math.random()*0xffffffff|0;
        var d2 = Math.random()*0xffffffff|0;
        var d3 = Math.random()*0xffffffff|0;
        return lut[d0&0xff]+lut[d0>>8&0xff]+lut[d0>>16&0xff]+lut[d0>>24&0xff]+'-'+
            lut[d1&0xff]+lut[d1>>8&0xff]+'-'+lut[d1>>16&0x0f|0x40]+lut[d1>>24&0xff]+'-'+
            lut[d2&0x3f|0x80]+lut[d2>>8&0xff]+'-'+lut[d2>>16&0xff]+lut[d2>>24&0xff]+
            lut[d3&0xff]+lut[d3>>8&0xff]+lut[d3>>16&0xff]+lut[d3>>24&0xff];
    };
    return self;
};
